from itertools import count
import logging


import requests
from redis import StrictRedis


# to be redone, probably.
def wb_get(string="", force=False, params=None, expire=604800):
    base = "http://api.worldbank.org/{}?format=json&per_page=1000&page={}"
    if params is not None and isinstance(params, dict):
        base += "&" + "&".join(["{}={}".format(key, params[key]) for key in params])
    r = StrictRedis()
    key = "WB|{}".format(string)
    if not force and r.exists(key):
        return r.get(key)
    else:
        l = []
        try:
            for p in count(1):
                req = requests.get(base.format(string, p))
                j = req.json()
                req.close()
                pages = j[0]["pages"]
                # log.info("Got page {} of {}{}".format(p, pages, " ..." if p < pages else "!"))
                l.extend(j[1])
                if p >= pages:
                    break
            r.set(key, l)
            if expire is not None:
                r.expire(key, expire)
        except ValueError:
            return None
        return l



import pandas as pd

from .who import who_dataset, who_dims, who_get
from .oecd import oecd_dataset, oecd_dims, oecd_inds, oecd_md

import kleptes.goodies

__all__ = [
    "who_dataset", "who_dims", "who_get",
    "oecd_dataset", "oecd_dims", "oecd_inds",
    "pd"
]



import uuid

import pandas as pd


def _sbs(df, ascending=True, copy=True):
    """
    Sometimes you want a stacked bar plot and have the bars sorted by their sum: this is the case for labelled bar
    plots, which are stacked plots where only one bar has non-zero height, e.g. say you have a metric by country, each
    of which is labelled by a region/continent and you want to colour the bars by region/continent. Simply call
    df.sbs() on your multicolumn dataframe and you're done. If you only have one column this function will have no
    effect.

    :param df: a DataFrame (it must have one index only and, ideally, more columns)
    :param ascending: sort in ascending order (default: False)
    :param copy: get a copy or sort in-place (default: True)
    :return: your sorted DataFrame
    """
    if not isinstance(df, pd.DataFrame):
        raise TypeError("The argument must be a DataFrame")
    _df = df.copy() if copy else df
    c = uuid.uuid4()
    _df[c] = _df.sum(axis=1)
    _df = _df.sort_values(c, ascending=ascending)
    del _df[c]
    return _df


pd.DataFrame.sbs = _sbs



import os
from setuptools import setup

VERSION = "0.2.1"


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="kleptes",
    version=VERSION,
    author="myyc",
    description="Goodies to mine world data",
    license="BSD",
    keywords="data mining python jupyter who",
    packages=["kleptes"],
    long_description=read('README.md'),
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
    install_requires=["requests", "pandas", "redis", "bs4"],
)
